// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef BOARDEVALUATOR_H
#define BOARDEVALUATOR_H

class Board;

class BoardEvaluator
{
/**********************************************************************
* PUBLIC METHODS
*********************************************************************/
public:  
  BoardEvaluator() {};     

  // A simple, fast evaluation based exclusively on material.  Since material
  // is overwhelmingly more important than anything else, we assume that if a
  // position's material value is much lower (or much higher) than another,
  // then there is no need to waste time on positional factors because they
  // won't be enough to tip the scales the other way, so to speak.
  int EvaluateQuickie(Board& theBoard, int fromWhosePerspective);
    
  // A detailed evaluation function, taking into account several positional
  // factors
  int EvaluateComplete(Board& theBoard, int fromWhosePerspective);
  
/***************************************************************************
* PRIVATE METHODS
**************************************************************************/
private:  
  // All other things being equal, having your Knights, Queens and Rooks close
  // to the opponent's king is a good thing
  // This method is a bit slow and dirty, but it gets the job done
  int EvalKingTropism(Board& theBoard, int fromWhosePerspective);
  
    // Rooks are more effective on the seventh rank, on open files and behind
  // passed pawns
  int EvalRookBonus(Board& theBoard, int fromWhosePerspective);
  
  // Mostly useful in the opening, this term encourages the machine to move
  // its bishops and knights into play, to control the center with its queen's
  // and king's pawns, and to castle if the opponent has many major pieces on
  // the board
  int EvalDevelopment(Board& theBoard, int fromWhosePerspective);
  
  // If max has too many pawns on squares of the color of his surviving bishops,
  // the bishops may be limited in their movement
  int EvalBadBishops(Board& theBoard, int fromWhosePerspective);
  
    // Given the pawn formations, penalize or bonify the position according to
  // the features it contains
  int EvalPawnStructure(int fromWhosePerspective);
    
  // Look at pawn positions to be able to detect features such as doubled,
  // isolated or passed pawns
  bool AnalyzePawnStructure(Board& theBoard, int fromWhosePerspective);

/**********************************************************************
* DATA MEMBERS
**********************************************************************/

  // Data counters to evaluate pawn structure
  int MaxTotalPawns;
  int PawnRams;
  int MaxPawnFileBins[8];
  int MaxPawnColorBins[2];  
  int MaxMostAdvanced[8];
  int MaxPassedPawns[8];
  int MinPawnFileBins[8];
  int MinMostBackward[8];

  // The "graininess" of the evaluation.  MTD(f) works a lot faster if the
  // evaluation is relatively coarse
  static const int Grain = 3;
};

#endif
