// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef HANDSTATE_H
#define HANDSTATE_H

class Piece;
class Hand;
class AnimationBlender;

class HandState
{
public:
  // enums
  enum ActionType
  { Waiting, Move, Capture, EnPassant, Castle, Promotion, Win };

  enum StateType
  { Static, Pickup, MovePiece, SimpleReturn, Take, Remove, 
    CaptureReturn, Tap1, Tap2, Tap3, ThumbUp, NewPiece };

  // Creation/destruction
  HandState() {};
	HandState(Hand* hand, const ActionType action, 
            const int SourceSquare = -1, const int DestinationSquare = -1,
            const int CastlingSource = -1, const int CastlingDestination = -1, const String PromoType = "");
	virtual ~HandState();    

  // accessors
  StateType GetStateType(void) const {return mStateType;}
	
	StateType BaseUpdate(Real dt);   

  // holds pointers to the current movement track - lazy
  AnimationState* mMoveState; 

  // used for castling
  int mCastlingSource;
  int mCastlingDestination;
  bool mCastleFirst;

protected:	
  // useful functions
  void GetNextRemoveSlot(Real& x, Real& z);

  // overridable functions  
  void BaseCreateMovementTrack(void);
  void BaseStartAnimation(void);  
  virtual void AttachPiece(const String bonename, Piece* piece, SceneNode*& pieceNode);  
  virtual void DetachPiece(Piece* piece, SceneNode*& pieceNode);

  // data members
  Hand* mHand;  
  Real mStateDuration;

  // holds the 00-63 positions of the current move
  int mSourceSquare; 
  int mDestinationSquare;    

  // en passant stuff
  bool mEnPassantFirst;

  // used for promotion
  String mPromoType;

  // used to tweak track positioning
  Ogre::Vector3 mOffset;

  // holds the target position (in world coordinates) at the end of this current state's life
  Ogre::Vector3 mTargetPosition; 

  // sets where the hand goes between moves
  Ogre::Vector3 mStartingPosition;

  // this is used to help calculate the animation tracks properly
  Ogre::Vector3 mMovementVector;  

  // holds which action the hand should be performing
  ActionType mActionType;

  // holds which current state within the action the hand is at
  StateType mStateType;     

  // nabbed this to allow for smooth animation blending
  AnimationBlender* mAnimationBlender;  

  // we need to hold a pointer to the entities that will be attached to the hand
  // and also their original scene node so we can move it into position and re-attach when required.
  Piece* mMovingPiece;
  Piece* mCapturedPiece;
  SceneNode* mMPSceneNode;
  SceneNode* mCPSceneNode;   

  // some other pointers for quickness-sake
  SceneManager* mSceneManager;
  SceneNode* mHandNode;  
  Entity* mHandEntity;

};

#endif
