// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#ifndef PIECE_H
#define PIECE_H

class CollisionEntity;
class Board;

class Piece
{
public:
	// Creation/destruction
  Piece(const int colour, const String type, const int position, 
        const int rotation = 0, const String shortcode = "");
  ~Piece();	
  
  // accessors  
  SceneNode* GetSceneNode(void) {return mScenenode;}  
  int GetColour(void) {return mColour;}
  Entity* GetEntity(void) {return mEntity;}  
  CollisionEntity* GetCollisionEntity(void);
  
  int GetPosition(void) {return mPosition;};
  void SetPosition(const int position);
  void PromotePiece(const String type);  
  bool Update(const Real dt);    

  bool IsMoveValid(int destPosition);

  static Piece* GetPiece(const int position);
  static Piece* GetPiece(const String name);  

  void GetValidMoves(void);
 
  static void SetupPieces(Board* theBoard);      
  static void DestroyPieces(void);  
  
protected:     
  int mColour;
  int mPosition;  
  String mShortCode;          
  Ogre::Vector3 mOrigin;
  Ogre::Vector3 mDestination;              
  Entity *mEntity;
  CollisionEntity *mColEntity;
  SceneNode *mScenenode;    

private:        
  std::vector<int> mValidMoves;	
  static int mPieceNumber;
  static SceneManager *mSceneManager;
  static Piece *mPieces[32];        
};

#endif
