// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "CaptureState.h"
#include "AnimationBlender.h"
#include "Piece.h"
#include "Hand.h"

void CaptureState::Initialise(void) 
{ 
  // record this state type
  mStateType = HandState::Take;   

  // set how long it takes for this part of the move to complete
  mStateDuration = 1;   

  // put the hand in it's starting position
  CreateMovementTrack();      
}

void CaptureState::CreateMovementTrack(const int Type)
{    
  if (Type == 0)
  {
    // get the world coordinates of where the piece is to be moved to
    mTargetPosition = Ogre::Vector3::ZERO;
    Functions::ConvertCoords(mDestinationSquare, mTargetPosition.x, mTargetPosition.z);  

    // tweak the offsets
    mOffset.x = 17;
    mOffset.y = 50;
    mOffset.z = 50;

    // make sure the scene node's rotation is taken into account
    Ogre::Matrix3 nodeRot;      
    mHandNode->_getDerivedOrientation().ToRotationMatrix(nodeRot);
    mOffset = mOffset * nodeRot;

    // tweak the position according to the offsets
    mTargetPosition += mOffset;    

    // attach the moving piece to the hand
    AttachPiece("Index3", mMovingPiece, mMPSceneNode);

    // move the moving piece's node so it's in the right place when we reattach later
    mMPSceneNode->setPosition(mTargetPosition.x - mOffset.x, 0, mTargetPosition.z - mOffset.z);

    // store the movement vector (in world coordinates) 
    // of the target position from the entities current position
    mMovementVector = mTargetPosition - mHandNode->getPosition();  
        
    // create the path to be followed     
    Animation* anim = mSceneManager->createAnimation("HandTrack" + StringConverter::toString(mHand->GetColour()), mStateDuration);

    // Spline it for nice curves
    anim->setInterpolationMode(Animation::IM_SPLINE);

    // Create a track to animate the hand
    NodeAnimationTrack* track = anim->createNodeTrack(0, mHandNode);

    // Setup keyframes
    TransformKeyFrame* key = track->createNodeKeyFrame(0); // startposition
    key->setTranslate(mHandNode->getPosition());
    key->setRotation(mHandNode->getOrientation()); 

    key = track->createNodeKeyFrame(mStateDuration * 0.4);
    key->setTranslate(Ogre::Vector3(mTargetPosition.x - (mMovementVector.x * 0.1), mTargetPosition.y + 35, mTargetPosition.z - (mMovementVector.z * 0.1)));
    key->setRotation(mHandNode->getOrientation());
    
    key = track->createNodeKeyFrame(mStateDuration * 0.6);
    key->setTranslate(Ogre::Vector3(mTargetPosition.x, mTargetPosition.y + 25, mTargetPosition.z));
    key->setRotation(mHandNode->getOrientation());    

    key = track->createNodeKeyFrame(mStateDuration); 
    key->setTranslate(Ogre::Vector3(mTargetPosition.x - (mOffset.x * 0.78), mTargetPosition.y, mTargetPosition.z + (mOffset.z * 0.38))); // tweaked 'til it looks good
    key->setRotation(mHandNode->getOrientation()); 

    // start the animation
    StartAnimation(0);
  }
  else
  {    
    // start the animation
    StartAnimation(1);
  }
}

void CaptureState::StartAnimation(const int Type)
{   
  if (Type == 0)
  {
    // start the movement track
    HandState::BaseStartAnimation();
    // run the move animation
    mAnimationBlender->blend("Capture1", AnimationBlender::BlendWhileAnimating, mStateDuration * 0.25, false);   
  }
  else
    mAnimationBlender->blend("Capture2", AnimationBlender::BlendWhileAnimating, mStateDuration * 0.25, false);   
}

HandState::StateType CaptureState::Update(Real dt)
{   
  static bool Second = false;    
  static bool Third = false;    

  // cycle the animation    
  HandState::BaseUpdate(dt);

  // do the second part of the animation
  if (mMoveState->getTimePosition() >= 0.5 && !Second) 
  {    
    // set the second half of the animation
    CreateMovementTrack(1);    
    Second = true;
  } 

  if (mMoveState->getTimePosition() >= 0.6 && !Third) 
  {
    // attach the captured piece to the hand
    AttachPiece("Ring2", mCapturedPiece, mCPSceneNode);  
    Third = true;
  }

  if (mMoveState->getTimePosition() >= 1)
  {
    mSceneManager->destroyAnimation("HandTrack" + StringConverter::toString(mHand->GetColour()));
    mMoveState = 0;
    Second = false;
    Third = false;
    if (mPromoType == "")
      return HandState::Remove;
    else
      return HandState::NewPiece;
  }
  else    
    return HandState::Take;
}
