// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "ChessState.h"
#include "GameOGRE.h"
#include "GameAI.h"
#include "ChessApplication.h"

// State overrides
void ChessState::initialize()
{
	// Store pointer to Game instances
  mGameOGRE = GameOGRE::getSingletonPtr();  
  mGameAI = GameAI::getSingletonPtr();  
  mApplication = static_cast<ChessApplication*>(Application::getSingletonPtr());
}

void ChessState::mouseMoved(const MouseEvent & e)
{
  // Update CEGUI with the mouse motion  
  CEGUI::System::getSingleton().injectMouseMove(e.state.X.rel, e.state.Y.rel);
}

void ChessState::keyPressed(const KeyEvent & e)
{
	// Handle keys common to all game states
	switch (e.key)
	{
	case KC_F12:
		{		
			// Switch debug overlay
			mApplication->setShowDebugOverlay(!mApplication->getShowDebugOverlay());      
		}
		break;

	case KC_SYSRQ:
		{		
			// Take a screenshot
			mApplication->saveScreenshot();
		}
		break;
	}
}
void ChessState::frameStarted(Real dt)
{  
  // Run an AI loop
  mGameAI->Update();
  // Update game  
	mGameOGRE->Update(dt);
}

void ChessState::frameEnded(Real dt)
{
  // Update debug overlay
	mApplication->updateDebugOverlay();
}
