// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "GameMenuState.h"
#include "StateManager.h"
#include "CPUvCPUState.h"
#include "MainMenuState.h"

// Singleton instance
template<> GameMenuState* Singleton<GameMenuState>::ms_Singleton = 0;

GameMenuState::GameMenuState()
{
	OverlayManager* overlayManager = OverlayManager::getSingletonPtr();

	// Set this menu's overlay
	mMenuOverlay = static_cast<Overlay*>(OverlayManager::getSingleton().getByName("Menu/GameMenu"));

	// Add menu items  
	mMenuItems.push_back(overlayManager->getOverlayElement("GameMenu/ResumeGame"));
	mMenuItems.push_back(overlayManager->getOverlayElement("GameMenu/Resign"));  
}

// State overrides
void GameMenuState::keyPressed(const KeyEvent & e)
{
	if (e.key == KC_ESCAPE)
		StateManager::getSingleton().popState();
	else
		MenuState::keyPressed(e);
}

void GameMenuState::onSelected(MenuItemList::iterator item)
{
  // leave this menu
  StateManager::getSingleton().popState();

	if ((*item)->getName() == "GameMenu/Resign")
	{		
    // set it back to demo mode
    StateManager::getSingleton().pushState(CPUvCPUState::getSingletonPtr());    
    // Return to the main menu
	  changeState(MainMenuState::getSingletonPtr());
	}
}
