// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "InputManager.h"
#include "Application.h"
#include "Piece.h"
#include "ChessApplication.h"

// Singleton instance
template<> ChessInputManager* Singleton<ChessInputManager>::ms_Singleton = 0;

ChessInputManager::ChessInputManager(void) : mSelectionEntity(0), mSelectionNode(0), mCurrentEntity(0), mCurrentNode(0)
{
  // keep a pointer to the scene manager for quickness-sake
  mSceneManager = static_cast<Application*>(Application::getSingletonPtr())->getSceneManager();    

  // create/get an entity      
  mSelectionEntity = mSceneManager->createEntity("Selector", "Selector.mesh");            
  mSelectionEntity->setMaterialName("Chess/SelectionMat");  

  // create a plane to show the current mouse position    
  mPlane = new MovablePlane("CurrentMPlane");
  mPlane->d = 0;
  mPlane->normal = Ogre::Vector3::UNIT_Y;
  MeshManager::getSingleton().createPlane("CurrentPlane", 
      ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
      *mPlane, 20, 20, 
      1, 1, true, 1, 1, 1, Ogre::Vector3::UNIT_Z);
  mCurrentEntity = mSceneManager->createEntity("CurrentEntity", "CurrentPlane" );  

  // create the nodes to hang off          
  mSelectionNode = static_cast<ChessApplication*>(ChessApplication::getSingletonPtr())->GetGameRootNode()->createChildSceneNode("SelectorNode");           
  mSelectionNode->attachObject(mSelectionEntity);
  
  mCurrentNode = static_cast<ChessApplication*>(ChessApplication::getSingletonPtr())->GetGameRootNode()->createChildSceneNode("CurrentNode");             
  mCurrentNode->attachObject(mCurrentEntity);
    
  mSelectionNode->setVisible(false);
  mCurrentNode->setVisible(false);
}

ChessInputManager::~ChessInputManager()
{
  delete mPlane;

  // Destroy scene nodes
  if (mSelectionNode) {    
	  mSelectionNode->detachAllObjects();
    mSelectionNode->getParentSceneNode()->removeAndDestroyChild(mSelectionNode->getName());    
    mSelectionNode = 0;
  }

  if (mCurrentNode) {    
	  mCurrentNode->detachAllObjects();
    mCurrentNode->getParentSceneNode()->removeAndDestroyChild(mCurrentNode->getName());    
    mCurrentNode = 0;
  }
  // get rid of the entities
	if (mSelectionEntity) {
		mSceneManager->destroyEntity(mSelectionEntity);    
    mSelectionEntity = 0;
  }

  if (mCurrentEntity) {
		mSceneManager->destroyEntity(mCurrentEntity);    
    mCurrentEntity = 0;
  }
}

void ChessInputManager::ShowSelectionNode(SceneNode *node)
{
  // move the node
  mSelectionNode->setPosition(node->getPosition());  

  // make it visible
  mSelectionNode->setVisible(true);  

  // hide the movement node
  HideCurrentNode();
}

void ChessInputManager::HideSelectionNode(void)
{  
  mSelectionNode->setVisible(false);  
}

void ChessInputManager::ShowCurrentNode(Ogre::Vector3 position, Piece* thePiece, int& currentPosition)
{    
  Ogre::Vector3 showPos;
  int pos;

  if (!Functions::GetNearestPosition(position, showPos, pos))
    pos = -1;    
  else
  {
    // make sure there's not a piece of the same colour on this position
    Piece* testPiece = Piece::GetPiece(pos);
    if (testPiece != 0)
      if (thePiece->GetColour() == testPiece->GetColour())
        pos = -1;
  }

  // set which whether this is a legal move or not
  if (thePiece->IsMoveValid(pos))
    mCurrentEntity->setMaterialName("Chess/GreenSel");  
  else
    mCurrentEntity->setMaterialName("Chess/RedSel");  

  mCurrentNode->setPosition(showPos);

  // only show the node if we've moved away from the piece  
  mCurrentNode->setVisible(thePiece->GetPosition() != pos && pos != -1);  

  // return the converted position
  currentPosition = pos;
}

void ChessInputManager::HideCurrentNode(void)
{
  mCurrentNode->setVisible(false);
}

void ChessInputManager::Update(Real dt)
{
  // rotate the entity
  if (mSelectionNode != 0)
    mSelectionNode->yaw(Degree(-360 * dt), Node::TS_PARENT);
}
