// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "MenuState.h"
#include "ChessApplication.h"
#include "GameOGRE.h"

MenuState::MenuState() : mMenuOverlay(0) {}

// State overrides
void MenuState::initialize()
{
  ChessState::initialize();

  // hide the mouse pointer
  CEGUI::MouseCursor::getSingleton().hide();

	static_cast<ChessApplication*>(Application::getSingletonPtr())->setMenuOverlay(mMenuOverlay);

	mSelectedItem = mMenuItems.begin();

	for (MenuItemList::iterator i = mMenuItems.begin(); i != mMenuItems.end(); ++i)
		setItemState(i, i == mSelectedItem);
}

void MenuState::cleanup()
{
	static_cast<ChessApplication*>(Application::getSingletonPtr())->setMenuOverlay(0);
}

void MenuState::frameStarted(Real dt)
{
	// Slowly rotate level
	mGameOGRE->mCameraWantedYaw += Radian(0.25*dt).valueDegrees();
  while (Radian(mGameOGRE->mCameraWantedYaw).valueRadians() > 2.0*Math::PI)
    mGameOGRE->mCameraWantedYaw -= Radian(2.0*Math::PI).valueDegrees();

	ChessState::frameStarted(dt);
}

void MenuState::keyPressed(const KeyEvent &e)
{
	switch (e.key)
	{
	case KC_RETURN:
	case KC_NUMPADENTER:
		onSelected(mSelectedItem);
		break;

	default:
		// Default processing
		ChessState::keyPressed(e);
	}
}

void MenuState::keyRepeated(int kc)
{
	// Handle keys common to all menu states
	switch (kc)
	{
	case KC_UP:
		setItemState(mSelectedItem, false);
		if (mSelectedItem != mMenuItems.begin())
			--mSelectedItem;
		else
			mSelectedItem = mMenuItems.end() - 1;

		setItemState(mSelectedItem, true);
		break;

	case KC_DOWN:
		setItemState(mSelectedItem, false);
		if (++mSelectedItem == mMenuItems.end())
			mSelectedItem = mMenuItems.begin();

		setItemState(mSelectedItem, true);
		break;
	}
}

void MenuState::mouseMoved(const MouseEvent & e)
{
  ChessState::mouseMoved(e);
}

void MenuState::setItemState(MenuItemList::iterator item, bool selected)
{
	OverlayElement* element = *item;

	// Apply hardcoded menu item colours
	if (selected)
	{
		element->setParameter("colour_top", "1.0 1.0 0.0");
		element->setParameter("colour_bottom", "0.8 0.8 0.0");
	}
	else
	{
		element->setParameter("colour_top", "1.0 1.0 1.0");
		element->setParameter("colour_bottom", "0.8 0.8 0.8");
	}
}
