// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "Move.h"
#include "Board.h"

Move::Move(const Move& original) 
{
  Copy(original);
}

void Move::Copy(const Move& target)
{
  MovingPiece = target.MovingPiece;
  CapturedPiece = target.CapturedPiece;
  SourceSquare = target.SourceSquare;
  DestinationSquare = target.DestinationSquare;
  MoveType = target.MoveType;
  MoveEvaluation = target.MoveEvaluation;
  MoveEvaluationType = target.MoveEvaluationType;
  SearchDepth = target.SearchDepth;
}

// Check whether two Move objects contain the same data (not necessarily
// whether they are the same object in memory)
bool Move::Equals(const Move& target)
{
  if (MovingPiece != target.MovingPiece)
    return false;
  if (CapturedPiece != target.CapturedPiece)
    return false;
  if (MoveType != target.MoveType)
    return false;
  if (SourceSquare != target.SourceSquare)
    return false;
  if (DestinationSquare != target.DestinationSquare)
    return false;
  return true;
}

bool Move::Reset(void)
{
  MovingPiece = Board::EMPTY_SQUARE;
  CapturedPiece = Board::EMPTY_SQUARE;
  SourceSquare = NULL_MOVE;
  DestinationSquare = NULL_MOVE;
  MoveType = NULL_MOVE;
  MoveEvaluation = NULL_MOVE;
  MoveEvaluationType = NULL_MOVE;
  SearchDepth = NULL_MOVE;
  return true;
}