// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "PlayerHuman.h"
#include "Board.h"
#include "Move.h"
#include "MoveListGenerator.h"

Move PlayerHuman::GetMove(Board& theBoard, bool& Continue) 
{
  // should never hit this...    
  Move mov; 
  return mov;
} 

bool PlayerHuman::CompileMove(Board* theBoard, Move* Mov)
{
  bool OK = true;

  // Time to try to figure out what the move means!
  if ( theBoard->GetCurrentPlayer() == Player::SIDE_WHITE )
  {           
    Mov->MovingPiece = theBoard->FindWhitePiece( Mov->SourceSquare );
    Mov->CapturedPiece = theBoard->FindBlackPiece( Mov->DestinationSquare );
    if ( Mov->CapturedPiece != Board::EMPTY_SQUARE )
      Mov->MoveType = Move::MOVE_CAPTURE_ORDINARY;
    else if ( ( theBoard->GetEnPassantPawn() == ( 1 << Mov->DestinationSquare ) ) &&
      ( Mov->MovingPiece == Board::WHITE_PAWN ) )
    {
      Mov->CapturedPiece = Board::BLACK_PAWN;
      Mov->MoveType = Move::MOVE_CAPTURE_EN_PASSANT;
    }
    // If the move isn't a capture, it may be a castling attempt
    else if ( ( Mov->MovingPiece == Board::WHITE_KING ) &&
              ( ( Mov->SourceSquare - Mov->DestinationSquare ) == -2 ) )
      Mov->MoveType = Move::MOVE_CASTLING_KINGSIDE;
    else if ( ( Mov->MovingPiece == Board::WHITE_KING ) &&
              ( ( Mov->SourceSquare - Mov->DestinationSquare ) == 2 ) )
      Mov->MoveType = Move::MOVE_CASTLING_QUEENSIDE;
    else
      Mov->MoveType = Move::MOVE_NORMAL;
  }
  else
  {
    Mov->MovingPiece = theBoard->FindBlackPiece( Mov->SourceSquare );      
    Mov->CapturedPiece = theBoard->FindWhitePiece( Mov->DestinationSquare );
    if ( Mov->CapturedPiece != Board::EMPTY_SQUARE )
      Mov->MoveType = Move::MOVE_CAPTURE_ORDINARY;
    else if ( ( theBoard->GetEnPassantPawn() == ( 1 << Mov->DestinationSquare ) ) &&
              ( Mov->MovingPiece == Board::BLACK_PAWN ) )
    {
      Mov->CapturedPiece = Board::WHITE_PAWN;
      Mov->MoveType = Move::MOVE_CAPTURE_EN_PASSANT;
    }
    else if ( ( Mov->MovingPiece == Board::BLACK_KING ) &&
              ( ( Mov->SourceSquare - Mov->DestinationSquare ) == -2 ) )
              Mov->MoveType = Move::MOVE_CASTLING_KINGSIDE;
    else if ( ( Mov->MovingPiece == Board::BLACK_KING ) &&
              ( ( Mov->SourceSquare - Mov->DestinationSquare ) == 2 ) )
      Mov->MoveType = Move::MOVE_CASTLING_QUEENSIDE;
    else
      Mov->MoveType = Move::MOVE_NORMAL;
  }

  // Now, if the move results in a pawn promotion, we must ask the user
  // for the type of promotion!
  if ( ( ( Mov->MovingPiece == Board::WHITE_PAWN ) && ( Mov->DestinationSquare < 8 ) ) ||
        ( ( Mov->MovingPiece == Board::BLACK_PAWN ) && ( Mov->DestinationSquare > 55 ) ) )
  {
    OK = false; // we'll use this return value to display the promotion menu elsewhere
  }         

  return OK;
}

std::vector<int> PlayerHuman::GetValidMoves(Board& theBoard, int sourcePosition)
{
  std::vector<int> ValidMoves;

  // first, compute the list of moves possible for the whole board at this time
  bool Continue = true;
  Pseudos.ComputeLegalMoves(theBoard, Continue);
  
  // then, go through and filter out a list of moves possible for the piece at this source position
  std::vector<Move>::iterator testMove;
  while ((testMove = Pseudos.Next()) != Pseudos.End())  
  {
    if (testMove->SourceSquare == sourcePosition)
    {
      // make sure that moving this piece does not leave the king in check
      Successor.Clone(theBoard);
      Successor.ApplyMove(*testMove);
      if (Check.ComputeLegalMoves(Successor, Continue))    
        ValidMoves.push_back(testMove->DestinationSquare);
    }
  }

  return ValidMoves;
}