// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "PromotionState.h"
#include "Piece.h"
#include "Hand.h"

void PromotionState::Initialise(void) 
{ 
  // record this state type
  mStateType = HandState::NewPiece; 
  mStateDuration = 0;

  // set the initial animation
  StartAnimation();  
}

void PromotionState::StartAnimation(void)
{
  // we don't move anywhere in this state, so stop any movement tracks    
  mMoveState = 0;
}

HandState::StateType PromotionState::Update(Real dt)
{         
  // cycle the animation
  HandState::BaseUpdate(dt);  

  // snap the hand into it's final position to make sure
  mHandNode->setPosition(mTargetPosition);    
  // detach the piece from the hand
  DetachPiece(mMovingPiece, mMPSceneNode);
  // update the piece structure to its new position
  mMovingPiece->SetPosition(mDestinationSquare);
  // promote the piece
  mMovingPiece->PromotePiece(mPromoType);    
  // go to the next state    
  if (mActionType == HandState::Move ||
      mActionType == HandState::Castle ||
      mActionType == HandState::Promotion ||
      mActionType == HandState::EnPassant)
    return HandState::SimpleReturn;  
  else
    return HandState::Remove;
 
}