// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "StaticState.h"
#include "Hand.h"
#include "AnimationBlender.h"
#include "GameAI.h"

void StaticState::Initialise(void)
{ 
  // record this state type
  mStateType = HandState::Static; 
  mPromoType = "";

  // turn off particles
  mHand->ToggleParticle(false); 

  // put the hand in it's starting position
  // - make sure the scene node's rotation is taken into account
  Ogre::Matrix3 nodeRot;      
  mHandNode->_getDerivedOrientation().ToRotationMatrix(nodeRot);
  mTargetPosition = (mStartingPosition * nodeRot);  
  mHandNode->setPosition(mTargetPosition);   
}

void StaticState::StartAnimation()
{ 
  // choose one of the three thinking animations - sometimes none
  int Type = int(Math::RangeRandom(0, 4));

  if (Type > 0 && Type < 4)
    mAnimationBlender->blend("Tap" + StringConverter::toString(Type),  AnimationBlender::BlendWhileAnimating, 1.0 ); 
}

HandState::StateType StaticState::Update(Real dt)
{
  // cycle the animation - only do this for the current player
  if (GameAI::getSingletonPtr()->GetCurrentPlayerColour() == mHand->GetColour())
    HandState::BaseUpdate(dt);

  // when the animation has finished, choose another animation at random
  static Real time = 0;
  time += dt;
  if (time >= 3) // choose a new animation every 3 seconds
  {
    // StopAnimation();
    StartAnimation();
    time = 0;
  }

  return mStateType;
}
