// Chess 
// Copyright (C) 2005  Steve Driscoll (steviedisco@gmail.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "stdafx.h"
#include "TranspositionTable.h"
#include "Board.h"
#include "Move.h"

TranspositionTable::TranspositionTable() {};

TranspositionTable::~TranspositionTable() {};

// Store a good evaluation found through alphabeta for a certain board position
bool TranspositionTable::StoreBoard(Board& theBoard, int eval, int evalType, int depth, int timeStamp)
{
  int key = abs(theBoard.HashKey() % TABLE_SIZE);

  // Would we erase a more useful (i.e., higher) position if we stored this
  // one?  If so, don't bother!
  if ((Table[key].theEvalType != TranspositionEntry::NULL_ENTRY) &&
      (Table[key].theDepth > depth) &&
      (Table[key].timeStamp >= timeStamp))
    return true;

  // And now, do the actual work
  Table[key].theLock = theBoard.HashLock();
  Table[key].theEval = eval;
  Table[key].theDepth = depth;
  Table[key].theEvalType = evalType;
  Table[key].timeStamp = timeStamp;
  return true;
}

// Verify whether there is a stored evaluation for a given board.
// If so, return TRUE and copy the appropriate values into the
// output parameter
bool TranspositionTable::LookupBoard(Board& theBoard, Move& theMove)
{
  // Find the board's hash position in Table
  int key = abs(theBoard.HashKey() % TABLE_SIZE );
  TranspositionEntry* entry = &Table[key];

  // If the entry is an empty placeholder, we don't have a match
  if ( (*entry).theEvalType == TranspositionEntry::NULL_ENTRY )
    return false;

  // Check for a hashing collision!
  if ( (*entry).theLock != theBoard.HashLock() )
    return false;

  // Now, we know that we have a match!  Copy it into the output parameter
  // and return
  theMove.MoveEvaluation = (*entry).theEval;
  theMove.MoveEvaluationType = (*entry).theEvalType;
  theMove.SearchDepth = (*entry).theDepth;
  return true;
}
